Program azd;              {By Basil L. Copeland, Jr. - Release 1.0, 3/28/83}
                          {Source compiled with TP4                        }

                          (* AZD began as an unabashed hack of Phil Burns' *)
                          (* source to pibcat.  Not much remains except    *)
                          (* the record type declarations used to          *)
                          (* represent the arc and zip header structures.  *)
                          (* Then along came Lharc, and the LZH file       *)
                          (* format.  The data structure used here is      *)
                          (* tentative, until we see some published specs. *)
                          (* John Newlin shared his knowledge of the LZH   *)
                          (* file strucutre with me.                       *)

Uses dos,crt;

(*--------------------------------------------------------------------------*)
(*                          General Declarations                            *)
(*--------------------------------------------------------------------------*)

                                (* The type az_shortrec is used for holding *)
                                (* only as much header information we are   *)
                                (* interested in displaying.  We could, of  *)
                                (* course, construct the displays directly  *)
                                (* from the arc and zip header structures,  *)
                                (* but these routines were written with     *)
                                (* other uses in mind which call for doing  *)
                                (* it this way.                             *)
type

   az_shortrec   =  record

                      filename          : string[12];
                      size_o            : longint;
                      size_c            : longint;
                      datestr           : string[8];
                      timestr           : string[8];

                    end;

var

    az_array                        : array[0..99] of az_shortrec;
    az_file                         : file;
    az_pos,timedate                 : longint;
    bytes_read,i,j                  : integer;
    filename                        : string;
    timedate_w                      : array[1..2] of word absolute timedate;
    dt                              : datetime;
    format_error                    : boolean;
    total_azsize_o, total_azsize_c  : longint;
    ext                             : string[3];
    dirinfo                         : searchrec;

(*--------------------------------------------------------------------------*)
(*             Record Structure, Constants, and Vars for Arc Files          *)
(*--------------------------------------------------------------------------*)


type

   fnametype = array[1..13] of char;

   arcrec = record

                marker   : byte      (* Flags beginning of entry *);
                version  : byte      (* Compression method       *);
                filename : fnametype (* file and extension       *);
                size_c   : longint   (* Compressed size          *);
                date     : word      (* Packed date              *);
                time     : word      (* Packed time              *);
                crc      : word      (* Cyclic Redundancy Check  *);
                size_o   : longint   (* Original length          *);

             end;

const

   headerlength          = 29      (* Length of an archive header entry *);
   archive_marker        = 26      (* Marks start of an archive header  *);

var


   archeader     : arcrec               (* Header for one file in archive *);


(*--------------------------------------------------------------------------*)
(*         Record Strurcture, Constants, and Vars for Zip Files             *)
(*--------------------------------------------------------------------------*)

const

   Zip_Central_Header_Signature  = $02014B50;
   Zip_Local_Header_Signature    = $04034B50;
   Zip_End_Central_Dir_Signature = $06054B50;


type

   z_filename_type         = array[1..12] of char;

                               (*Structure of a local file header*)
   Zip_Local_Header_Type =

      record

         signature          : longint  (* Header signature        *);
         version            : word     (* Vers. needed to extract *);
         bitflag            : word     (* General flags           *);
         compression_method : word     (* Compression type used   *);
         time               : word     (* File creation time      *);
         date               : word     (* File creation date      *);
         crc32              : longint  (* 32-bit CRC of file      *);
         size_c             : longint  (* Compressed size of file *);
         size_o             : longint  (* Original size of file   *);
         filename_length    : word     (* Length of file name     *);
         extrafield_length  : word     (* Length of extra stuff   *);

      end;
                                   (* Structure of the central *)
                                   (* directory record         *)
   Zip_Central_Header_Type =

      record

          signature           : longint (* Header signature        *);
          version_made_by     : word    (* System id/program vers. *);
          version             : word    (* Vers. needed to extract *);
          bitflag             : word    (* General flags           *);
          compression_method  : word    (* Compression type used   *);
          time                : word    (* File creation time      *);
          date                : word    (* File creation date      *);
          crc32               : longint (* 32-bit CRC of file      *);
          size_c              : longint (* Compressed size of file *);
          size_o              : longint (* Original size of file   *);
          filename_length     : word    (* Length of file name     *);
          extrafield_length   : word    (* Length of extra stuff   *);
          commentfield_length : word    (* Length of comments      *);
          diskstart_umber     : word    (* Disk # file starts on   *);
          internal_attributes : word    (* Text/non-text flags     *);
          external_attributes : longint (* File system attributes  *);
          localheader_offset  : longint (* Where local hdr starts  *);
      end;

var

   c_zipheader   : Zip_Central_Header_Type (* Central header       *);
   l_zipheader   : zip_local_header_type;
   File_Name     : fnametype;       (* File name of entry in Zip file  *)


(*----------------------------------------------------------------------*)
(*        Record Structure for LZH files (still pretty tenative)        *)
(*----------------------------------------------------------------------*)

type

   lzhrec =

      record

         reclen             : byte;
         unknown1           : byte;
         id                 : word;       {we have five bytes here, which }
         unknown2           : word;       {yield the string '-lh1-'       }
         unknown3           : byte;       {                               }
         size_c             : longint;
         size_o             : longint;
         time               : word;
         date               : word;
         attr               : word;       {maybe two fields here          }
         filename_length    : byte;

      end;

                                 (*The filename comes at the end of   *)
                                 (*this record, and is of variable    *)
                                 (*length.  It is then followed by    *)
                                 (*the CRC.  This implementation does *)
                                 (*not try and get the CRC.           *)

var

    lzhheader : lzhrec;

(*------------------------------------------------------------------------*)
(*                        Miscellaneous Routines                          *)
(*------------------------------------------------------------------------*)

function lower(str:string):string;
var
  i : integer;
begin
  for i := 1 to length(str) do
    if ord(str[i]) in [65..90] then
           str[i] := chr(ord(str[i]) + 32);
  lower := str;
end;


function withzeros(a_number : integer) : string ;

var  wz : string ;                       {reformats date and time fields  }
                                         {as two-byte strings with zeroes }
begin
  str(a_number:2,wz) ;
  if wz[1] = ' ' then wz[1] := '0' ;
  if wz[2] = ' ' then wz[2] := '0' ;
  withzeros := wz ;

end;


procedure convert_az_datetime;

begin

   unpacktime(timedate,dt);

   az_array[j].timestr := withzeros(dt.hour)+':'
                        + withzeros(dt.min)+':'
                        + withzeros(dt.sec);

   az_array[j].datestr := withzeros(dt.month)+'/'
                        + withzeros(dt.day)+'/'
                        + withzeros(dt.year-1900);

end;

procedure display_az_array;

begin
     writeln;
     writeln('Contents of: ',dirinfo.name);
     writeln;
     writeln('  FileName     Length     Size      Date      Time');
     writeln('------------  --------  --------  --------  --------');

     for i := 0 to j-2 do

         writeln(lower(az_array[i].filename):12,'  ',
                 az_array[i].size_o:8,'  ',
                 az_array[i].size_c:8,'  ',
                 az_array[i].datestr:8,'  ',
                 az_array[i].timestr:8);

     for i := 0 to j-2 do

        begin
          total_azsize_o := total_azsize_o + az_array[i].size_o;
          total_azsize_c := total_azsize_c + az_array[i].size_c;
        end;

     writeln('------------  --------  --------  --------  --------');
     writeln('Total bytes:  ',total_azsize_o:8,'  ',total_azsize_c:8);
     writeln;

end;

procedure init_az;

begin

  reset(az_file,1);
  az_pos:=0; j:=0;
  format_error := false;
  total_azsize_o :=0;
  total_azsize_c :=0;

end;

procedure next_header;

begin

  seek(az_file,az_pos);
  j:=j+1;

end;

procedure usage;

begin

  writeln;
  writeln('Usage:  azd <somefile.ext>');
  writeln;
  writeln('Valid extentions are: ARC, ZIP, and LZH.');
  writeln;
  writeln('Normal wildcards (e.g. *.LZH) will return the');
  writeln('contents of all matches.');
  writeln;

end;

(*------------------------------------------------------------------------*)
(*                         Procedure ArcScan                              *)
(*------------------------------------------------------------------------*)

                                      (*This is the routine that scans an *)
                                      (*ARC file and constructs the array *)
                                      (*for displaying the contents.      *)


procedure arcscan;

var

   fnstr: string[13];

begin

  init_az;

  repeat

    blockread(az_file,archeader, headerlength, bytes_read);
    if archeader.marker <> 26 then format_error := true;
    fnstr := archeader.filename;                 {gets rid of leftover chars}
    for i:= pos(chr(0),fnstr) to 13
      do archeader.filename[i] := chr(0);

    az_array[j].filename := archeader.filename;
    az_array[j].size_o := archeader.size_o;
    az_array[j].size_c := archeader.size_c;

    timedate_w[1] := archeader.time;
    timedate_w[2] := archeader.date;
    convert_az_datetime;
                                            {now we jump to the next header}

    az_pos := az_pos + archeader.size_c + headerlength;
    next_header;

until (bytes_read = 0) or (bytes_read < headerlength)
    or (format_error = true) or (archeader.version = 0);

end; {arcscan}

(*------------------------------------------------------------------------*)
(*                         Procedure ZipScan                              *)
(*------------------------------------------------------------------------*)

                                      (*This is the routine that scans a  *)
                                      (*ZIP file and constructs the array *)
                                      (*for displaying the contents.      *)

procedure zipscan;

begin

  init_az;

  repeat
    blockread(az_file,l_zipheader, sizeof(l_zipheader), bytes_read);
    if l_zipheader.signature <> zip_local_header_signature
      then format_error := true;
    fillchar(file_name,sizeof(file_name),32);
                                               {in a zip file the filename }
                                               {comes at the end of the    }
                                               {header, so we get the file }
                                               {name length and read in    }
                                               {the filename               }

    blockread(az_file,file_name,l_zipheader.filename_length,bytes_read);

    timedate_w[1] := l_zipheader.time;
    timedate_w[2] := l_zipheader.date;
    convert_az_datetime;

    az_array[j].filename := file_name;
    az_array[j].size_o := l_zipheader.size_o;
    az_array[j].size_c := l_zipheader.size_c;

    az_pos:=az_pos+sizeof(l_zipheader)+l_zipheader.size_c
          +l_zipheader.filename_length+l_zipheader.extrafield_length;
    next_header;
                                         {zip files have dual headers for  }
                                         {each file: a "local" header and  }
                                         {a central header at the end of   }
                                         {the file.  we've been getting    }
                                         {everything from the local        }
                                         {headers, so we can stop when     }
                                         {we get to the central directory. }

                                         {note that if you want access to  }
                                         {the comment fields, you have to  }
                                         {access the central directory. in }
                                         {that case you should skip the    }
                                         {local headers, and get everything}
                                         {from the central directory.      }

  until (l_zipheader.signature = zip_central_header_signature)
    or (format_error);

end; {zipscan}

(*------------------------------------------------------------------------*)
(*                         Procedure LzhScan                              *)
(*------------------------------------------------------------------------*)

                                      (*This is the routine that scans an *)
                                      (*LZH file and constructs the array *)
                                      (*for displaying the contents.      *)

procedure lzhscan;

begin

  init_az;

  repeat
    blockread(az_file,lzhheader, sizeof(lzhheader), bytes_read);

                                   {i'm sure that when published specs   }
                                   {are available, we can make some more }
                                   {sense out of the "id" field.         }
    if lzhheader.id <> $6c2d
      then format_error := true;

    fillchar(file_name,sizeof(file_name),32);

                                  {like in a zip file, we read the file  }
                                  {name in after the header structure.   }
                                  {this leaves out the crc, which oddly  }
                                  {comes after the filename.  if we want }
                                  {the crc, we'll have to read in another}
                                  {word.                                 }

    blockread(az_file,file_name,lzhheader.filename_length,bytes_read);

    timedate_w[1] := lzhheader.time;
    timedate_w[2] := lzhheader.date;
    convert_az_datetime;

    az_array[j].filename := file_name;

    az_array[j].size_o := lzhheader.size_o;
    az_array[j].size_c := lzhheader.size_c;

                                       {here we jump to the next header. }
                                       {the extra 2 bytes takes account  }
                                       {of the crc field which our       }
                                       {record skips over.               }

    az_pos:=az_pos+lzhheader.size_c
          +sizeof(lzhheader) +lzhheader.filename_length+2;

    next_header;

  until (lzhheader.reclen = 0)
    or (format_error);

end; {zipscan}

(*-----------------------------------------------------------------------*)
(*                              Main Program                             *)
(*-----------------------------------------------------------------------*)

                                      (*There's no sophisticated command *)
                                      (*tail parsing or error checking   *)
                                      (*here.  This is just meant to be  *)
                                      (*a simple illustration, so don't  *)
                                      (*expect too much.  What the       *)
                                      (*program expects is that you will *)
                                      (*pass valid paths and filenames   *)
                                      (*to it, and nothing more.         *)

begin {main program}

  if paramstr(1) <> '' then

    begin

      ext := '   ';
      filename := paramstr(1);
      ext[1] := filename[length(filename)-2];
      ext[2] := filename[length(filename)-1];
      ext[3] := filename[length(filename)];

      findfirst(paramstr(1),$37,dirinfo);

      if doserror <> 0 then
        begin
          writeln;
          writeln('File not found.');
          writeln;
        end;

      while doserror = 0 do

        begin

          assign(az_file,dirinfo.name);

          if ext = 'arc' then
            begin
              arcscan;
              display_az_array;
            end

          else if ext = 'zip' then
            begin
              zipscan;
              display_az_array;
            end

          else if ext = 'lzh' then
            begin
              lzhscan;
              display_az_array;
            end;

          findnext(dirinfo);

          close(az_file);

        end;

    end

    else usage;

end.